#include "crmc2ndst.h"

#include <EventFile.h>
#include <Event.h>
using namespace evt61;

#include <NA61Detector.h>
using namespace det61;

#include <TDatabasePDG.h>

#include <iostream>
#include <sstream>
#include <vector>
#include <string>
#include <iomanip>
using namespace std;

#include <TDirectory.h>

/*************************************************************
*
*  main()
*
*     run with command line option -h to get a list of options
*
**************************************************************/

int main(int argc, char **argv) {

  // read options from comand line

  if (!GetOptions(argc,argv)) return 1;

  // initialize ROOT output file

  rootOut(-1);

  // open FORTRAN IO at first call

  const char * conexRoot = gSystem->Getenv("CONEX_ROOT");
  if ( conexRoot == NULL ) {
     conexRoot = gSystem->Getenv("PWD");
  }

  crmc_init_f_(gSeed,gBeamMomentum,gTargetMomentum,gBeamID,
               gTargetID,gHEModel,gParamFileName.c_str());

  std::cout.precision(10);

  for (int iColl = 0; iColl < gNCollision; ++iColl){

    // cleanup vectors
    cleanVector();

    if (iColl % 1000 == 0)
      cout << " ==[crmc]==> Collision number " << iColl+1 << endl;

    // loop over collisions
      crmc_f_(gNParticles,gImpactParameter,
              gPartID[0],gPartPx[0],gPartPy[0],gPartPz[0],
              gPartEnergy[0],gPartMass[0]);

    rootOut(0);

  }

  std::cout.precision(2);

  rootOut(1);

  cout << " \n succesfully processed " << gNCollision << " collision"
       << ( gNCollision > 1 ? "s \n" : " \n" ) ;

  gStopwatch.Stop();
  const double realTime = gStopwatch.RealTime();
  const double cpuTime = gStopwatch.CpuTime();

  cout << " in " << realTime << " sec, "
       << ((double)realTime/gNCollision) << " sec/collision, with "
       << (cpuTime/realTime*100) << "% cpu usage. \n" << endl;
  return 0;
}


/*************************************************************
*
*   GetOptions()  - reads command line options
*
**************************************************************/
bool GetOptions(int argc, char **argv){
  int c;
  ostringstream help;
  help << "      -m [0=EPOS LHC, 1=EPOS 1.99"
#ifdef __QGSJET01__
       << ", 2=QGSJET01"
#endif
#ifdef __GHEISHA__
       << ", 3=Gheisha"
#endif
#ifdef __PYTHIA__
       << ", 4=Pythia 6.115"
#endif
#ifdef __HIJING__
       << ", 5=Hijing 1.38"
#endif
#ifdef __SIBYLL__
       << ", 6=Sibyll 2.1"
#endif
#ifdef __QGSJETII__
       << ", 7=QGSJETII-03"
#endif
#ifdef __Phojet__
       << ", 8=Phojet"
#endif
       << "]" << std::endl;
  help << "      -p [beam's momentum/(GeV/c)]" << std::endl;
  help << "      -P [target's momentum/(GeV/c)" << std::endl;
  help << "      -b [beam's id:   1=p, 12=C, 120=pi+, 208=Pb, pdg ID]" << std::endl;
  help << "      -t [target's id: 1=p, 12=C, 208=Pb, pdg nuc ID]" << std::endl;
  help << "      -c [path/filename of parameter/setup file]" << std::endl;
  help << "      -n [Number of events to simulate]" << std::endl;
  help << "      -x [prefix]" << std::endl;

  while ((c = getopt (argc, argv, "s:m:p:P:b:t:B:c:n:x:h")) != -1)
    switch (c)
      {
      case 's':
        gSeed=atoi(optarg);
        break;
      case 'm':
        gHEModel=atoi(optarg);
        break;
      case 'p':
        gBeamMomentum=atof(optarg);
        break;
      case 'p':
        gTargetMomentum=atof(optarg);
        break;
      case 'n':
        gNCollision=atoi(optarg);
        break;
      case 'b':
        gBeamID=atoi(optarg);
        break;
      case 't':
        gTargetID=atoi(optarg);
        break;
      case 'c':
        gParamFileName=optarg;
        break;
      case 'x':
        gPrefix=optarg;
        break;
      case 'h':
        cout << help.str() << endl;
        return false;
      default:
        cout << help.str() << endl;
        return false;
      }

  cout << "\n          >> crmc <<\n\n"
       << "  prefix:                     " << gPrefix << "\n"
       << "  beam'id:                    " << gBeamID << "\n"
       << "  beam's momentum:            " << gBeamMomentum << "\n"
       << "  target's id:                " << gTargetID << "\n"
       << "  target's momentum:          " << gTargetMomentum << "\n\n";

  cout << "  number of collisions:       " << gNCollision << "\n"
       << "  parameter file name:        " << gParamFileName << "\n"
       << "  HE model:                   " << gHEModel;
  if ( gHEModel == 0 )
    cout << " (EPOS LHC) \n";
  else if ( gHEModel == 1 )
    cout << " (EPOS 1.99) \n";
  else if ( gHEModel == 2 )
    cout << " (QGSJET01) \n";
  else if ( gHEModel == 3 )
    cout << " (Gheisha)\n ";
  else if ( gHEModel == 4 )
    cout << " (Pythia)\n ";
  else if ( gHEModel == 5 )
    cout << " (Hijing)\n ";
  else if ( gHEModel == 6 )
    cout << " (Sibyll)\n ";
  else if ( gHEModel == 7 )
    cout << " (QGSJETII) \n";
  else if ( gHEModel == 8 )
    cout << " (Phojet) \n";
  else
    cout << " (unknown) \n";
  cout << endl;

  cout.setf(ios::showpoint);
  cout.setf(ios::fixed);
  cout.precision(3);
  return true;
}

/*************************************************************
*
*   rootOut(int iopt)
*
*   opens ROOT file (iopt=-1), stores the data (iopt=0)
*   and closes ROOT file (iopt=1)
*
**************************************************************/
void rootOut(int iopt){

  static EventFile * outputFile;
  static Event * na61Event;
  static TDatabasePDG * particleCodes = TDatabasePDG::Instance();
  static TVector3 * mainVertexPos;

  if ( iopt==-1 ) { // open file and define tree

    // check if ROOT_OUT is set (if not assume PWD)

    const char * rootOutDir = gSystem->Getenv("ROOT_OUT");
    if ( rootOutDir == NULL ) {
      rootOutDir = gSystem->Getenv("PWD");
    }

    // open ROOT output file

    ostringstream rootFileName;
    rootFileName << rootOutDir << "/" << gPrefix << "_";
    switch (gHEModel) {
    case 0: rootFileName << "eposlhc";      break;
    case 1: rootFileName << "epos";      break;
    case 2: rootFileName << "qgsjet";    break;
    case 3: rootFileName << "gheisha";   break;
    case 4: rootFileName << "pythia";    break;
    case 5: rootFileName << "hijing";    break;
    case 6: rootFileName << "sibyll";    break;
    case 7: rootFileName << "qgsjetII";  break;
    case 8: rootFileName << "phojet";    break;
    default:
      cout << " rootOut: error - unknown model " << gHEModel << endl;
      cout << "          exit ..." << endl;
      exit(1);
      break;
    }

    rootFileName << "_" << gSeed << "_";

    switch (gBeamID) {
    case 120  : rootFileName << "pi+"; break;
    case -120 : rootFileName << "pi+"; break;
    case 1    : rootFileName << "p";   break;
    case 12   : rootFileName << "C";   break;
    case 207  : rootFileName << "Pb";  break;
    default:
      cout << " rootOut: error - unkown beam's ID" << gBeamID << endl;
      cout << "          exit ..." << endl;
      exit(1);
      break;
    }

    switch (gTargetID) {
    case 1    : rootFileName << "p";   break;
    case 12   : rootFileName << "C";   break;
    case 207  : rootFileName << "Pb";  break;
    default:
      cout << " rootOut: error - unkown target's ID" << gTargetID << endl;
      cout << "          exit ..." << endl;
      exit(1);
      break;
    }

    rootFileName << "_" << gBeamMomentum << "_ndst.root";

    outputFile = new EventFile(rootFileName.str().c_str(),EventFile::eWrite);
    na61Event = new Event();
    outputFile->SetBuffers(&na61Event);

    det61::NA61Detector & theDetector = det61::NA61Detector::GetInstance();
    VDetector& target = theDetector.GetDetector("Target");
    mainVertexPos = new TVector3(0.,0.,target.GetPosition().Z());
  }
  else if  ( iopt==0 ) {

    // Clear event
    na61Event->Clear();

    SimEvent & simEvent = na61Event->GetSimEvent();
    //simEvent.SetImpactParameter(gImpactParameter);

    SimVertex vertex(*mainVertexPos,0.);

    for (int iPart = 0; iPart < gNParticles; ++iPart){
      const int geantID = particleCodes->ConvertPdgToGeant3(gPartID[iPart]);
      const int charge = particleCodes->GetParticle(gPartID[iPart])->Charge()/3;

      TVector3 point2(0.,0.,0.);
      TVector3 momentum(gPartPx[iPart],gPartPy[iPart],gPartPz[iPart]);
      SimTrack simTrack(*mainVertexPos,point2,momentum,gPartEnergy[iPart],
        		charge,geantID);
      vertex.AddTrack(simTrack);

    }
    simEvent.AddVertex(vertex);
    outputFile->WriteEvent();

  }
  else {

    outputFile->Close();

  }
}
/*************************************************************
*
*   cleanVector()
*
*   clean vectors defined in eposcc.h
*
**************************************************************/

void cleanVector(){

  for (int iPart = 0; iPart < gNParticles; ++iPart){
    gPartID[iPart] = 0;
    gPartPx[iPart] = 0.0;
    gPartPy[iPart] = 0.0;
    gPartPz[iPart] = 0.0;
    gPartEnergy[iPart] = 0.0;
    gPartMass[iPart] = 0.0;
  }

}

